library ieee;
use ieee.std_logic_1164.all;



entity CONTROLUNIT is
 port (clk : in std_logic;
       rst : in std_logic;
       opcod : in std_logic_vector(5 downto 0);
       MEMRead,MEMWrite : out std_logic;
       PCWrite,PCWriteCond, IorD, IRWrite, MemtoReg : out std_logic;
       TargetWrite,ALUSelA, RegWrite, RegDest: out std_logic;
       ALUOp: out std_logic_vector(1 downto 0);
       PCSource, ALUSelB : out std_logic_vector(1 downto 0);
       AWrite,BWrite, DataMemWrite,AluOutputWrite: out std_logic ;
       state: out std_logic_vector(3 downto 0)
      );       
end CONTROLUNIT;

architecture BEHAVIOR of CONTROLUNIT is
 signal stato : std_logic_vector(3 downto 0):="XXXX";
begin
  
  state <= stato;
  
 process(clk,rst)
 begin
  if rst='1' then stato <= "0000";
  elsif clk'event and clk = '0' then
    case stato is
     when "0000" => stato <= "0001"; -- fetch -> decode
     when "0001" => case opcod is
                 	
                 	when "000000" => stato <= "0010"; --dec to ex R 
                 	when "000001" => stato <= "1110"; --dec to ex add immediato
                 	when "010000" => stato <= "0100"; --dec to ex branch
                 	when "100000"|"101000"  => stato <= "1000"; -- dec to -> ex lw/sw
                	when "000100"           => stato <= "0110"; -- dec to jump
                	when others => stato <= "0000";
   	            end case; 
     when "1000" => if opcod(3) = '0' 
                      then stato <= "1010"; -- ex lw/sw to mem read
                      else stato <= "1100"; -- ex lw/sw to mem write
                end if;
     when "1010" => stato <= "1011"; -- mem read to write back

     when "0010" => stato <= "0011"; -- ex R to write back
     when "1110" => stato <= "1111"; -- ex Immediato to write back
     when "0011"|"0100"|"0110"|"1011"|"1100"| "1111" => stato <= "0000"; -- to fetch
     when others => null;
     
    end case;
  end if;
  
 end process;   

process(stato)
 begin
 case stato is
  -- fetch delle istruzioni
   when "0000" => 
     		  IorD <= '0';         -- sel PC
     		  MEMRead <= '1';     -- lettura in memoria
             	  MEMWrite<= '0';   
             	  PCWrite <= '1';      -- scrive su PC
             	  IRWrite <= '1';      -- scrivi su IR
             	  RegWrite <= '0';     
             	  ALUSelA <= '0';      -- PC
             	  ALUSelB <= "01";     -- +4
             	  ALUop <= "00";      -- add 
             	  PCSource <= "00";    -- sel PC+4
             	  TargetWrite <= '0';  
             	  PCWriteCond <='0';
             	  AWrite <='0';
             	  BWrite <= '0';
             	  AluOutputWrite <= '0';  
             	  DataMemWrite <= '0';
             	  MemtoReg <= '0';
             	  RegDest <= '0';
             

   -- decodifica
   when "0001" => IorD <= '0'; 
   		  MEMRead <= '0';
   		  MEMWrite<= '0';
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '0';
             	  ALUSelA <= '0';      -- PC
             	  ALUSelB <= "11";     -- estensione di istruzione(15 downto 0)
             	  ALUop <= "00";      -- add 
     		  PCSource <= "00";    
             	  TargetWrite <= '1';  -- scrittura su Target
  		  PCWriteCond <= '0';
  		  AWrite <='1';     -- legge RegA e scrive in A
             	  BWrite <= '1';    -- legge RegB e scrive in B
             	  DataMemWrite <= '0';
             	  AluOutputWrite <= '0';  
             	  MemtoReg <= '0';
             	  RegDest <= '0';
             
  
    -- calcolo degli indirizzi per memory access
   when "1000" => 
                  IorD <= '0'; 
   		  MEMRead <= '0';
   		  MEMWrite<= '0';
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '0';
             	  ALUSelA <= '1';      -- A
              	  ALUSelB <= "10";     -- estensione di istruzione(15 downto 0)
             	  ALUop <= "00";      -- add 
     		  PCSource <= "00";    
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '0';
  		  AWrite <='0';     
             	  BWrite <= '0';    
             	  DataMemWrite <= '0';
             	  AluOutputWrite <= '1'; -- scrive il l'indirizzo di memoria  
             	  MemtoReg <= '0';
             	  RegDest <= '0';
             	  
   -- memory read access
   when "1010" => 
                  IorD <= '1';          -- sel indirizzo come risultatoALU
              	  MEMRead <= '1';	-- seleziona memoria in lettura
   		  MEMWrite<= '0';
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '0';
             	  ALUSelA <= '1';      
              	  ALUSelB <= "10";    
             	  ALUop <= "00";       
     		  PCSource <= "00";    
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '0';
  		  AWrite <='0';     
             	  BWrite <= '0'; 
             	  DataMemWrite <= '1';  -- scrive il dato letto dalla memoria  
             	  AluOutputWrite <= '0';  
             	  MemtoReg <= '0';
             	  RegDest <= '0';
             	  
   -- write back dato letto dalla memoria
   when "1011" => 
                  IorD <= '1';         
              	  MEMRead <= '0';	
   		  MEMWrite<= '0';
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '1';  -- scrittura nel Reg File
             	  ALUSelA <= '1';      
              	  ALUSelB <= "10";    
             	  ALUop <= "00";       
     		  PCSource <= "00";    
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '0';
  		  AWrite <='0';     
             	  BWrite <= '0'; 
             	  DataMemWrite <= '0';    
             	  AluOutputWrite <= '0';  
             	  MemtoReg <= '1'; -- seleziona dato da MemData
             	  RegDest <= '0'; -- seleziona registro dest rt
   
   
   -- memory write access
   when "1100" =>                   
                  IorD <= '1';     -- sel indirizzo come risultatoALU
              	  MEMRead <= '0';	
   		  MEMWrite<= '1' after 1 ns;  -- seleziona memoria in scrittura
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '0';
             	  ALUSelA <= '1';      
              	  ALUSelB <= "10";    
             	  ALUop <= "00";       
     		  PCSource <= "00";    
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '0';
  		  AWrite <='0';     
             	  BWrite <= '0'; 
             	  DataMemWrite <= '0';   
             	  AluOutputWrite <= '0';  
             	  MemtoReg <= '0';
             	  RegDest <= '0'; 
  
    --esecuzione  R 
   when "0010" => 
                  IorD <= '0'; 
   		  MEMRead <= '0';
   		  MEMWrite<= '0';
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '0';
             	  ALUSelA <= '1';      -- A
             	  ALUSelB <= "00";     -- B
             	  ALUop <= "10";      -- add 
     		  PCSource <= "00";    
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '0';
  		  AWrite <='0';     
             	  BWrite <= '0';    
             	  DataMemWrite <= '0';
             	  AluOutputWrite <= '1';  -- scrive risultato Alu  
             	  MemtoReg <= '0';
             	  RegDest <= '0';
             
                
                
   -- Write back R 
   when "0011" =>   
    		  IorD <= '0'; 
   		  MEMRead <= '0';
   		  MEMWrite<= '0';
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '1';  -- scittura su Reg File
               	  ALUSelA <= '1';      
             	  ALUSelB <= "00";     
             	  ALUop <= "10";       
     		  PCSource <= "00";    
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '0';
  		  AWrite <='0';     
             	  BWrite <= '0';    
             	  DataMemWrite <= '0';
             	  AluOutputWrite <= '0';    
             	  MemtoReg <= '0';  -- sel risultatoALU
             	  RegDest <= '1';    -- sel registro dest rd        
  
   --esecuzione  Immediato 
   when "1110" => 
                  IorD <= '0'; 
   		  MEMRead <= '0';
   		  MEMWrite<= '0';
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '0';
             	  ALUSelA <= '1';      -- A
             	  ALUSelB <= "10";     -- Immediato esteso in segno
             	  ALUop <= "00";      -- add 
     		  PCSource <= "00";    
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '0';
  		  AWrite <='0';     
             	  BWrite <= '0';    
             	  DataMemWrite <= '0';
             	  AluOutputWrite <= '1';  -- scrive risultato Alu  
             	  MemtoReg <= '0';
             	  RegDest <= '0';
             
                
                
   -- Write back Immediato 
   when "1111" =>   
    		  IorD <= '0'; 
   		  MEMRead <= '0';
   		  MEMWrite<= '0';
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '1';  -- scittura su Reg File
               	  ALUSelA <= '1';      
             	  ALUSelB <= "00";     
             	  ALUop <= "10";       
     		  PCSource <= "00";    
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '0';
  		  AWrite <='0';     
              	  BWrite <= '0';    
             	  DataMemWrite <= '0';
             	  AluOutputWrite <= '0';    
             	  MemtoReg <= '0';  -- sel risultatoALU
             	  RegDest <= '0';    -- sel registro dest rd        
  
   -- conclusione branch
   when "0100" => 
                  IorD <= '0'; 
   		  MEMRead <= '0';
   		  MEMWrite<= '0';
             	  PCWrite <= '0';
             	  IRWrite <= '0';     
             	  RegWrite <= '0';
             	  ALUSelA <= '1';      -- A
              	  ALUSelB <= "00";     -- B
             	  ALUop <= "01";      -- sub 
     		  PCSource <= "01";   -- sel Target;
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '1';  -- salto condizionato 
  		  AWrite <='0';     
             	  BWrite <= '0';    
             	  DataMemWrite <= '0';
             	  AluOutputWrite <= '0';   
             	  MemtoReg <= '0';
             	  RegDest <= '0';
             	  
   -- conclusione jmp
   when "0110" => 
                  IorD <= '0'; 
   		  MEMRead <= '0';
   		  MEMWrite<= '0';
             	  PCWrite <= '1';   -- aggiorna PC
             	  IRWrite <= '0';     
             	  RegWrite <= '0';
             	  ALUSelA <= '1';      
              	  ALUSelB <= "00";    
             	  ALUop <= "00";       
     		  PCSource <= "10";   -- sel Jump;
             	  TargetWrite <= '0';  
  		  PCWriteCond <= '0';   
  		  AWrite <='0';     
             	  BWrite <= '0';    
             	  DataMemWrite <= '0';
             	  AluOutputWrite <= '0';   
             	  MemtoReg <= '0';
             	  RegDest <= '0';
             	  
     when others => null;
  end case;
 end process; 
 
 
end BEHAVIOR;














