library ieee;
use ieee.std_logic_1164.all;
use work.dati.all;




entity datapath is
 port (clk : in std_logic;
       rst : in std_logic;
       datain : in w32;
       MemAddr : out w32;
       dataout :out w32;
       opcod : out std_logic_vector(5 downto 0);
       
       PCWrite,PCWriteCond, IorD, IRWrite, MemtoReg : in std_logic;
       TargetWrite,ALUSelA, RegWrite, RegDest: in std_logic;
       AluOpCrtl: in std_logic_vector(1 downto 0);
       PCSource, ALUSelB : in std_logic_vector(1 downto 0);
       AWrite,BWrite, DataMemWrite,AluOutputWrite: in std_logic 
       
       );
end datapath;

architecture schematic of datapath is

component  registro is
port (d : in w32;
        clk : in std_logic;
        rst : in std_logic;
        q : out w32
        );
end component;

component MUX2V32 is
    port (
        a: in w32;
        b: in std_logic_vector(31 downto 0);
        sel: in std_logic;
        q: out w32
        
        );
end component;

component MUX2V5 is
    port (
        a: in std_logic_vector(4 downto 0);
        b: in std_logic_vector(4 downto 0);
        sel: in std_logic;
        q: out std_logic_vector(4 downto 0)
        
    );

end component;

component REGISTERFILE is
    port (
      regl1: in std_logic_vector(4 downto 0);
      regl2: in std_logic_vector(4 downto 0);
      regw: in std_logic_vector(4 downto 0);
      datow: in w32;
      RegWrite: in std_logic;
      rst : in std_logic;
      clk : in std_logic;
      datol1: out w32;
      datol2: out w32
        );
end component;

component ESTENSIONE32 is
    port (
           a: in w16;
           a32: out w32;
           ash32: out w32
         );

end component;

component MUX4V32 is
    port (
         r: in w32;
         c: in w32;
         i: in w32;
         s: in w32;
         sel : in std_logic_vector(1 downto 0);
         q: out w32
         );
end component;

component ALUCONTROL is
    port (
        ALUop: in std_logic_vector(1 downto 0);
        codfunc: in std_logic_vector(5 downto 0);
        op: out std_logic_vector(2 downto 0)
    );

end component;

component  ALU is
    port (
        a: in w32;
        b: in w32;
        op: in std_logic_vector(2 downto 0);
        ris: out w32;
        zero: out std_logic
       );

end component;


-- segnali di controllo
signal control_ir: std_logic;                   -- controllo su IR
signal istruzione: w32;                                  -- valore di uscita IR 
signal control_pc : std_logic;          -- controllo sul PC
signal control_datamem: std_logic; -- controllo su DataMem
signal datamem: w32; --valore registro DataMem
signal NPC,PCout: w32;                                   -- ingresso e uscita di PC
signal control_ra, control_rb: std_logic; -- controllo registri A e B
signal datoa,datob: w32;   --valori dei registri A e B
signal ALUop: std_logic_vector(2 downto 0);
signal a,b: w32;  -- ingressi ALU
signal risultatoALU: w32;
signal Zero: std_logic; -- valori prodotti dall'ALU
signal control_ALUout: std_logic; -- controllo AluOutput
signal ALUres: w32; -- valore di ALUOutput
signal addr : w32;                                       -- indirizzo della memoria
signal imm32,imm32sh : w32;                     -- immediato esteso a 32 bit e shiftato a sinistra di 2 bit;
signal regl1,regl2,regw: std_logic_vector(4 downto 0); -- registri di lettura e scrittura;

signal datol1,datol2,datow : w32; -- lettura dati 1 e 2 e scrittura 
signal control_Target: std_logic; -- controllo su registro Target
signal addrTarget : w32; -- indirizzo Target prossima istruzione
signal addrJmp : w32;  -- indirizzo di jump

signal funct : std_logic_vector(5 downto 0);
begin 



-- Program Counter
control_pc <= clk and ( PCWrite OR (PCWriteCond and Zero));
pc : registro   port map(NPC, control_pc, rst, PCout); 

-- Mux indirizzi per la memoria    
muxmemoria: mux2v32
   port map (PCout, ALUres, IorD, MemAddr);


-- Registro Dati letti da lw
control_datamem <= clk and DataMemWrite;
DatMem: registro     port map (datain, control_datamem, rst, datamem);
      
-- Instruction Register  
control_ir <= clk and IRWrite; 
IR : registro   port map(datain, control_ir, rst, istruzione); 

-- Codice operativo per la CONTROL UNIT
opcod <= istruzione(31 downto 26);

-- funzione dell'Alu
funct <= istruzione(5 downto 0);

-- registro lettura 1    
regl1 <= istruzione(25 downto 21);

-- registro lettura 1    
regl2 <= istruzione(20 downto 16);

-- Mux Registro scrittura
muxregWR: mux2v5
     port map (istruzione(20 downto 16), istruzione(15 downto 11), RegDest, regw);

-- Mux Scrittura dati
muxdatiWR: mux2v32
   port map (ALUres, datamem, MemtoReg, datow);

-- Register File         
regfile : registerfile
  port map ( regl1, regl2, regw, datow, RegWrite, rst,clk, datol1, datol2);

-- Estensione immediato
est: estensione32
   port map (istruzione(15 downto 0), imm32,imm32sh);

-- Reg A
control_ra <= clk and AWrite;
RA: registro      port map (datol1,control_ra, rst, datoa);

-- Reg B
control_rb <= clk and BWrite;
 RB: registro     port map (datol2,control_rb, rst, datob);
     
-- Mux ingresso a ALU
mux_a_ALU : mux2v32
  port map (PCout, datol1, ALUSelA, a);

    
-- Mux ingresso b ALU
mux_b_ALU : mux4v32
  port map (datol2, "00000000000000000000000000000100", imm32, imm32sh, ALUSelB, b);

--ALU Control
ALUCRTL: ALUCONTROL
  port map (AluOpCrtl, funct, ALUop);  
-- ALU
ALUCmp : ALU 
  port map (a, b, ALUop, risultatoALU, Zero);  

--ALU Output
control_ALUout <= clk and AluOutputWrite;
AluOut: registro     port map ( risultatoALU, control_ALUout, rst, ALUres);

-- Reg Target
control_Target <= clk and TargetWrite;
RegTarget : registro   port map ( risultatoALU, control_Target, rst, addrTarget);

-- Jmp address
addrjmp(31 downto 28) <= PCout(31 downto 28);
addrjmp(27 downto 2) <= istruzione(25 downto 0);
addrjmp(1 downto 0) <= "00";

-- mux selezione prossimo indirizzo
muxaddr: mux4v32
   port map (risultatoALU, addrTarget, addrjmp, addrjmp,PCSource,NPC);


dataout <= datol2;

  
end schematic;








